#include <Python.h>
#include <thread>
#include <stdlib.h>
#include <vector>

void ThreadFunc()
{
   PyGILState_STATE gstate;

   // importing module in this thread
   gstate = PyGILState_Ensure();

   PyObject* mod = PyImport_ImportModule( "crash_test" );
   PyObject* func = PyObject_GetAttrString( mod, "DoTest" );

   PyGILState_Release( gstate );

   for( ; ; )
   {
      // first section of code: taking GIL and calling DoTest() function in crash_test module
      gstate = PyGILState_Ensure();
      PyObject* callback = PyObject_CallObject( func, nullptr );
      PyGILState_Release( gstate );

      usleep( rand() % 10 );

      // second section of code: taking GIL again and calling callback, returned by invokation of DoTest()
      // then destroing the callback object
      gstate = PyGILState_Ensure();
      PyObject* res = PyObject_CallObject( callback, nullptr );
      Py_DECREF( res );
      Py_DECREF( callback );
      PyGILState_Release( gstate );
   }
}

int main()
{
    Py_Initialize();
    PyEval_InitThreads();
    // allowing multithreaded execution and launch C++ threads
    Py_BEGIN_ALLOW_THREADS

    std::vector< std::thread > threads;
    for( size_t i = 0 ; i < 10 ; ++i )
       threads.emplace_back( &ThreadFunc );
    // we will never join with any of these threads, since after some period
    // of execution app will crash
    for( auto& t : threads )
       t.join();

    Py_END_ALLOW_THREADS

    Py_FinalizeEx();

    return 0;
}
